#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX 			(0x1000)

#define CHAR2NUM(ch)	(ch-0x30)
#define NUM2CHAR(num)	(num+0x30)

typedef struct s_num
{
	//数值
	char* n;
	//右向左起始位置
	char* p;
	//数值长度
	int len;
	//存储空间大小
	int size;
} s_num;

//计算用到的中间变量
s_num temp0;
s_num temp1;
s_num temp2;

void s_init(s_num* num, char* val)
{
	num->len = strlen(val);
	num->size = MAX;
	num->n = malloc(num->size);
	num->p = num->n + num->len - 1;
	memcpy(num->n, val, num->len + 1);
}

void s_start()
{
	s_init(&temp0, "0");
	s_init(&temp1, "0");
	s_init(&temp2, "0");
}

void s_stop()
{
	free(temp0.n);
	free(temp1.n);
	free(temp2.n);
}

void s_cpy(s_num* tar, s_num* src)
{
	memcpy(tar->n, src->n, src->size);
	tar->len = src->len;
	tar->p = tar->n + tar->len - 1;
}

void s_clear()
{
	temp0.len = 1;
	temp0.n[0] = '0';
	temp0.n[1] = 0;
	temp0.p = temp0.n + temp0.len - 1;

	temp1.len = 1;
	temp1.n[0] = '0';
	temp1.n[1] = 0;
	temp1.p = temp1.n + temp1.len - 1;

	temp2.len = 1;
	temp2.n[0] = '0';
	temp2.n[1] = 0;
	temp2.p = temp2.n + temp2.len - 1;
}

//加法
int s_add(s_num* tar, s_num* src0, s_num *src1)
{
	if (tar == NULL || src0 == NULL || src1 == NULL)
	{
		return -1;
	}
	char* p0 = src0->p;
	char* p1 = src1->p;
	char* pt = tar->n;
	char en = 0;
	int length = src0->len > src1->len ? src0->len : src1->len;
	//循环计算加法
	for (int i = 0; i < length; i++)
	{
		char n0 = *p0;
		char n1 = *p1;
		if (i >= src0->len)
		{
			n0 = '0';
		}
		if (i >= src1->len)
		{
			n1 = '0';
		}

		//计算和
		char res = CHAR2NUM(n0) + CHAR2NUM(n1) + en;
		//计算进位的值
		en = res / 10;
		//当前位结果
		*pt = NUM2CHAR(res % 10);

		p0--;
		p1--;
		pt++;
	}

	int len = length;
	//处理最高位进位值
	if (en != 0)
	{
		*pt = NUM2CHAR(en);
		pt++;
		len++;
	}
	*pt = 0;

	//高低位反转位置
	for (int i = 0; i < len / 2; i++)
	{
		char t = tar->n[i];
		tar->n[i] = tar->n[len - 1 - i];
		tar->n[len - 1 - i] = t;
	}

	tar->len = len;
	tar->p = tar->n + tar->len - 1;
	return 0;
}

//乘法
int s_mul(s_num* tar, s_num* src0, s_num *src1)
{
	if (tar == NULL || src0 == NULL || src1 == NULL)
	{
		return -1;
	}
	s_clear();
	if (src1->len < src0->len)
	{
		s_num* t = src0;
		src0 = src1;
		src1 = t;
	}

	char* p0 = src0->p;
	char* p1 = src1->p;

	for (int i = 0; i < src0->len; i++)
	{
		char en = 0;
		char n0 = *p0;
		char* pt = temp0.n;
		char* p1 = src1->p;

		//乘法
		for (int j = 0; j < src1->len; j++)
		{
			char n1 = *p1;
			//计算乘法结果
			char res = CHAR2NUM(n0) * CHAR2NUM(n1) + en;
			//计算进位
			en = res / 10;
			*pt = NUM2CHAR(res % 10);
			p1--;
			pt++;
		}
		int len = src1->len;
		if (en != 0)
		{
			*pt = NUM2CHAR(en);
			pt++;
			len++;
		}
		*pt = 0;
		for (int e = 0; e < len / 2; e++)
		{
			char t = temp0.n[e];
			temp0.n[e] = temp0.n[len - 1 - e];
			temp0.n[len - 1 - e] = t;
		}
		temp0.len = len;
		temp0.p = temp0.n + temp0.len - 1;

		for (int s = 0; s < i; s++)
		{
			//temp1 *= 10
			temp0.p++;
			*(temp0.p) = '0';
			*(temp0.p + 1) = 0;
			temp0.len++;
		}

		//累加和
		s_add(&temp2, &temp0, &temp1);
		s_cpy(&temp1, &temp2);

		p0--;
	}

	s_cpy(tar, &temp1);

	return 0;
}

int main(int argc, char *argv)
{
	s_num n;
	s_num mut;
	s_num res;
	s_num inc;
	s_num sum;

	s_init(&n, "1");
	s_init(&mut, "1");
	s_init(&res, "0");
	s_init(&inc, "1");
	s_init(&sum, "0");

	s_start();
	// 100!
	for (int i = 0; i < 1000; i++)
	{
		//计算n的阶乘
		s_mul(&res, &mut, &n);
		s_cpy(&mut, &res);
		//将n加1
		s_add(&sum, &inc, &n);
		s_cpy(&n, &sum);
	}

	printf("%s\n", res.n);

	s_stop();
}
